import datetime
import os

from django.core.files.base import ContentFile
from django.core.management import call_command
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.timezone import now

from ..models import Attachment, Email, STATUS


class CommandTest(TestCase):
    def test_cleanup_mail_with_orphaned_attachments(self):
        self.assertEqual(Email.objects.count(), 0)
        email = Email.objects.create(to=['to@example.com'], from_email='from@example.com', subject='Subject')

        email.created = now() - datetime.timedelta(31)
        email.save()

        attachment = Attachment()
        attachment.file.save('test.txt', content=ContentFile('test file content'), save=True)
        email.attachments.add(attachment)
        attachment_path = attachment.file.name

        # We have orphaned attachment now
        call_command('cleanup_mail', days=30)
        self.assertEqual(Email.objects.count(), 0)
        self.assertEqual(Attachment.objects.count(), 1)

        # Actually cleanup orphaned attachments
        call_command('cleanup_mail', '-da', days=30)
        self.assertEqual(Email.objects.count(), 0)
        self.assertEqual(Attachment.objects.count(), 0)

        # Check that the actual file has been deleted as well
        self.assertFalse(os.path.exists(attachment_path))

        # Check if the email attachment's actual file have been deleted
        Email.objects.all().delete()
        email = Email.objects.create(to=['to@example.com'], from_email='from@example.com', subject='Subject')
        email.created = now() - datetime.timedelta(31)
        email.save()

        attachment = Attachment()
        attachment.file.save('test.txt', content=ContentFile('test file content'), save=True)
        email.attachments.add(attachment)
        attachment_path = attachment.file.name

        # Simulate that the files have been deleted by accidents
        os.remove(attachment_path)

        # No exceptions should break the cleanup
        call_command('cleanup_mail', '-da', days=30)
        self.assertEqual(Email.objects.count(), 0)
        self.assertEqual(Attachment.objects.count(), 0)

    def test_cleanup_mail(self):
        """
        The ``cleanup_mail`` command deletes mails older than a specified
        amount of days
        """
        self.assertEqual(Email.objects.count(), 0)

        # The command shouldn't delete today's email
        email = Email.objects.create(from_email='from@example.com', to=['to@example.com'])
        call_command('cleanup_mail', days=30)
        self.assertEqual(Email.objects.count(), 1)

        # Email older than 30 days should be deleted
        email.created = now() - datetime.timedelta(31)
        email.save()
        call_command('cleanup_mail', days=30)
        self.assertEqual(Email.objects.count(), 0)

    TEST_SETTINGS = {
        'BACKENDS': {
            'default': 'django.core.mail.backends.dummy.EmailBackend',
        },
        'BATCH_SIZE': 1,
    }

    @override_settings(POST_OFFICE=TEST_SETTINGS)
    def test_send_queued_mail(self):
        """
        Ensure that ``send_queued_mail`` behaves properly and sends all queued
        emails in two batches.
        """
        # Make sure that send_queued_mail with empty queue does not raise error
        call_command('send_queued_mail', processes=1)

        Email.objects.create(from_email='from@example.com', to=['to@example.com'], status=STATUS.queued)
        Email.objects.create(from_email='from@example.com', to=['to@example.com'], status=STATUS.queued)
        call_command('send_queued_mail', processes=1)
        self.assertEqual(Email.objects.filter(status=STATUS.sent).count(), 2)
        self.assertEqual(Email.objects.filter(status=STATUS.queued).count(), 0)

    def test_successful_deliveries_logging(self):
        """
        Successful deliveries are only logged when log_level is 2.
        """
        email = Email.objects.create(from_email='from@example.com', to=['to@example.com'], status=STATUS.queued)
        call_command('send_queued_mail', log_level=0)
        self.assertEqual(email.logs.count(), 0)

        email = Email.objects.create(from_email='from@example.com', to=['to@example.com'], status=STATUS.queued)
        call_command('send_queued_mail', log_level=1)
        self.assertEqual(email.logs.count(), 0)

        email = Email.objects.create(from_email='from@example.com', to=['to@example.com'], status=STATUS.queued)
        call_command('send_queued_mail', log_level=2)
        self.assertEqual(email.logs.count(), 1)

    def test_failed_deliveries_logging(self):
        """
        Failed deliveries are logged when log_level is 1 and 2.
        """
        email = Email.objects.create(
            from_email='from@example.com', to=['to@example.com'], status=STATUS.queued, backend_alias='error'
        )
        call_command('send_queued_mail', log_level=0)
        self.assertEqual(email.logs.count(), 0)

        email = Email.objects.create(
            from_email='from@example.com', to=['to@example.com'], status=STATUS.queued, backend_alias='error'
        )
        call_command('send_queued_mail', log_level=1)
        self.assertEqual(email.logs.count(), 1)

        email = Email.objects.create(
            from_email='from@example.com', to=['to@example.com'], status=STATUS.queued, backend_alias='error'
        )
        call_command('send_queued_mail', log_level=2)
        self.assertEqual(email.logs.count(), 1)



import logging
from logging.config import dictConfig


# Taken from https://github.com/nvie/rq/blob/master/rq/logutils.py
def setup_loghandlers(level=None):
    # Setup logging for post_office if not already configured
    logger = logging.getLogger('post_office')
    if not logger.handlers:
        dictConfig(
            {
                'version': 1,
                'disable_existing_loggers': False,
                'formatters': {
                    'post_office': {
                        'format': '[%(levelname)s]%(asctime)s PID %(process)d: %(message)s',
                        'datefmt': '%Y-%m-%d %H:%M:%S',
                    },
                },
                'handlers': {
                    'post_office': {'level': 'DEBUG', 'class': 'logging.StreamHandler', 'formatter': 'post_office'},
                },
                'loggers': {'post_office': {'handlers': ['post_office'], 'level': level or 'DEBUG'}},
            }
        )
    return logger



from django.db import models, migrations
import post_office.validators
import post_office.fields


class Migration(migrations.Migration):
    dependencies = [
        ('post_office', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailtemplate',
            options={'verbose_name': 'Email Template', 'verbose_name_plural': 'Email Templates'},
        ),
        migrations.AddField(
            model_name='email',
            name='backend_alias',
            field=models.CharField(default='', max_length=64, blank=True),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='default_template',
            field=models.ForeignKey(
                related_name='translated_templates',
                default=None,
                to='post_office.EmailTemplate',
                null=True,
                on_delete=models.deletion.SET_NULL,
            ),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='language',
            field=models.CharField(
                default='',
                help_text='Render template in alternative language',
                max_length=12,
                blank=True,
                choices=[
                    ('af', 'Afrikaans'),
                    ('ar', 'Arabic'),
                    ('ast', 'Asturian'),
                    ('az', 'Azerbaijani'),
                    ('bg', 'Bulgarian'),
                    ('be', 'Belarusian'),
                    ('bn', 'Bengali'),
                    ('br', 'Breton'),
                    ('bs', 'Bosnian'),
                    ('ca', 'Catalan'),
                    ('cs', 'Czech'),
                    ('cy', 'Welsh'),
                    ('da', 'Danish'),
                    ('de', 'German'),
                    ('el', 'Greek'),
                    ('en', 'English'),
                    ('en-au', 'Australian English'),
                    ('en-gb', 'British English'),
                    ('eo', 'Esperanto'),
                    ('es', 'Spanish'),
                    ('es-ar', 'Argentinian Spanish'),
                    ('es-mx', 'Mexican Spanish'),
                    ('es-ni', 'Nicaraguan Spanish'),
                    ('es-ve', 'Venezuelan Spanish'),
                    ('et', 'Estonian'),
                    ('eu', 'Basque'),
                    ('fa', 'Persian'),
                    ('fi', 'Finnish'),
                    ('fr', 'French'),
                    ('fy', 'Frisian'),
                    ('ga', 'Irish'),
                    ('gl', 'Galician'),
                    ('he', 'Hebrew'),
                    ('hi', 'Hindi'),
                    ('hr', 'Croatian'),
                    ('hu', 'Hungarian'),
                    ('ia', 'Interlingua'),
                    ('id', 'Indonesian'),
                    ('io', 'Ido'),
                    ('is', 'Icelandic'),
                    ('it', 'Italian'),
                    ('ja', 'Japanese'),
                    ('ka', 'Georgian'),
                    ('kk', 'Kazakh'),
                    ('km', 'Khmer'),
                    ('kn', 'Kannada'),
                    ('ko', 'Korean'),
                    ('lb', 'Luxembourgish'),
                    ('lt', 'Lithuanian'),
                    ('lv', 'Latvian'),
                    ('mk', 'Macedonian'),
                    ('ml', 'Malayalam'),
                    ('mn', 'Mongolian'),
                    ('mr', 'Marathi'),
                    ('my', 'Burmese'),
                    ('nb', 'Norwegian Bokmal'),
                    ('ne', 'Nepali'),
                    ('nl', 'Dutch'),
                    ('nn', 'Norwegian Nynorsk'),
                    ('os', 'Ossetic'),
                    ('pa', 'Punjabi'),
                    ('pl', 'Polish'),
                    ('pt', 'Portuguese'),
                    ('pt-br', 'Brazilian Portuguese'),
                    ('ro', 'Romanian'),
                    ('ru', 'Russian'),
                    ('sk', 'Slovak'),
                    ('sl', 'Slovenian'),
                    ('sq', 'Albanian'),
                    ('sr', 'Serbian'),
                    ('sr-latn', 'Serbian Latin'),
                    ('sv', 'Swedish'),
                    ('sw', 'Swahili'),
                    ('ta', 'Tamil'),
                    ('te', 'Telugu'),
                    ('th', 'Thai'),
                    ('tr', 'Turkish'),
                    ('tt', 'Tatar'),
                    ('udm', 'Udmurt'),
                    ('uk', 'Ukrainian'),
                    ('ur', 'Urdu'),
                    ('vi', 'Vietnamese'),
                    ('zh-cn', 'Simplified Chinese'),
                    ('zh-hans', 'Simplified Chinese'),
                    ('zh-hant', 'Traditional Chinese'),
                    ('zh-tw', 'Traditional Chinese'),
                ],
            ),
        ),
        migrations.AlterField(
            model_name='email',
            name='bcc',
            field=post_office.fields.CommaSeparatedEmailField(verbose_name='Bcc', blank=True),
        ),
        migrations.AlterField(
            model_name='email',
            name='cc',
            field=post_office.fields.CommaSeparatedEmailField(verbose_name='Cc', blank=True),
        ),
        migrations.AlterField(
            model_name='email',
            name='from_email',
            field=models.CharField(
                max_length=254, verbose_name='Email From', validators=[post_office.validators.validate_email_with_name]
            ),
        ),
        migrations.AlterField(
            model_name='email',
            name='html_message',
            field=models.TextField(verbose_name='HTML Message', blank=True),
        ),
        migrations.AlterField(
            model_name='email',
            name='message',
            field=models.TextField(verbose_name='Message', blank=True),
        ),
        migrations.AlterField(
            model_name='email',
            name='subject',
            field=models.CharField(max_length=255, verbose_name='Subject', blank=True),
        ),
        migrations.AlterField(
            model_name='email',
            name='to',
            field=post_office.fields.CommaSeparatedEmailField(verbose_name='Email To', blank=True),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='content',
            field=models.TextField(
                blank=True, verbose_name='Content', validators=[post_office.validators.validate_template_syntax]
            ),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='html_content',
            field=models.TextField(
                blank=True, verbose_name='HTML content', validators=[post_office.validators.validate_template_syntax]
            ),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='subject',
            field=models.CharField(
                blank=True,
                max_length=255,
                verbose_name='Subject',
                validators=[post_office.validators.validate_template_syntax],
            ),
        ),
        migrations.AlterUniqueTogether(
            name='emailtemplate',
            unique_together={('language', 'default_template')},
        ),
    ]



import datetime

from django.core.management.base import BaseCommand
from django.utils.timezone import now

from ...utils import cleanup_expired_mails


class Command(BaseCommand):
    help = 'Place deferred messages back in the queue.'

    def add_arguments(self, parser):
        parser.add_argument(
            '-d', '--days', type=int, default=90, help='Cleanup mails older than this many days, defaults to 90.'
        )

        parser.add_argument('-da', '--delete-attachments', action='store_true', help='Delete orphaned attachments.')

        parser.add_argument('-b', '--batch-size', type=int, default=1000, help='Batch size for cleanup.')

    def handle(self, verbosity, days, delete_attachments, batch_size, **options):
        # Delete mails and their related logs and queued created before X days
        cutoff_date = now() - datetime.timedelta(days)
        num_emails, num_attachments = cleanup_expired_mails(cutoff_date, delete_attachments, batch_size)
        msg = 'Deleted {0} mails created before {1} and {2} attachments.'
        self.stdout.write(msg.format(num_emails, cutoff_date, num_attachments))



from collections import OrderedDict
from email.mime.base import MIMEBase
from django.core.files.base import ContentFile
from django.core.mail.backends.base import BaseEmailBackend
from .settings import get_default_priority


class EmailBackend(BaseEmailBackend):
    def open(self):
        pass

    def close(self):
        pass

    def send_messages(self, email_messages):
        """
        Queue one or more EmailMessage objects and returns the number of
        email messages sent.
        """
        from .mail import create
        from .models import STATUS, Email
        from .utils import create_attachments
        from .signals import email_queued

        if not email_messages:
            return

        default_priority = get_default_priority()
        num_sent = 0
        emails = []
        for email_message in email_messages:
            subject = email_message.subject
            from_email = email_message.from_email
            headers = email_message.extra_headers
            if email_message.reply_to:
                reply_to_header = ', '.join(str(v) for v in email_message.reply_to)
                headers.setdefault('Reply-To', reply_to_header)
            message = email_message.body  # The plaintext message is called body
            html_body = ''  # The default if no html body can be found
            if hasattr(email_message, 'alternatives') and len(email_message.alternatives) > 0:
                for alternative in email_message.alternatives:
                    if alternative[1] == 'text/html':
                        html_body = alternative[0]

            attachment_files = {}
            for attachment in email_message.attachments:
                if isinstance(attachment, MIMEBase):
                    attachment_files[attachment.get_filename()] = {
                        'file': ContentFile(attachment.get_payload()),
                        'mimetype': attachment.get_content_type(),
                        'headers': OrderedDict(attachment.items()),
                    }
                else:
                    attachment_files[attachment[0]] = ContentFile(attachment[1])

            email = create(
                sender=from_email,
                recipients=email_message.to,
                cc=email_message.cc,
                bcc=email_message.bcc,
                subject=subject,
                message=message,
                html_message=html_body,
                headers=headers,
            )

            if attachment_files:
                attachments = create_attachments(attachment_files)

                email.attachments.add(*attachments)

            emails.append(email)

            if default_priority == 'now':
                status = email.dispatch()
                if status == STATUS.sent:
                    num_sent += 1

        if default_priority != 'now':
            email_queued.send(sender=Email, emails=emails)

        return num_sent



from django.utils.html import mark_safe, format_html
from django.utils.translation import gettext_lazy

try:
    import bleach
except ImportError:
    # if bleach is not installed, render HTML as escaped text to prevent XSS attacks
    heading = gettext_lazy("Install 'bleach' to render HTML properly.")
    clean_html = lambda body: format_html('<p><em>{heading}</em></p>\n<div>{body}</div>', heading=heading, body=body)
else:
    styles = [
        'border',
        'border-top',
        'border-right',
        'border-bottom',
        'border-left',
        'border-radius',
        'box-shadow',
        'height',
        'margin',
        'margin-top',
        'margin-right',
        'margin-bottom',
        'margin-left',
        'padding',
        'padding-top',
        'padding-right',
        'padding-bottom',
        'padding-left',
        'width',
        'max-width',
        'min-width',
        'border-collapse',
        'border-spacing',
        'caption-side',
        'empty-cells',
        'table-layout',
        'direction',
        'font',
        'font-family',
        'font-style',
        'font-variant',
        'font-size',
        'font-weight',
        'letter-spacing',
        'line-height',
        'text-align',
        'text-decoration',
        'text-indent',
        'text-overflow',
        'text-shadow',
        'text-transform',
        'white-space',
        'word-spacing',
        'word-wrap',
        'vertical-align',
        'color',
        'background',
        'background-color',
        'background-image',
        'background-position',
        'background-repeat',
        'bottom',
        'clear',
        'cursor',
        'display',
        'float',
        'left',
        'opacity',
        'outline',
        'overflow',
        'position',
        'resize',
        'right',
        'top',
        'visibility',
        'z-index',
        'list-style-position',
        'list-style-tyle',
    ]
    tags = [
        'a',
        'abbr',
        'acronym',
        'b',
        'blockquote',
        'br',
        'caption',
        'center',
        'code',
        'em',
        'div',
        'font',
        'h1',
        'h2',
        'h3',
        'h4',
        'h5',
        'h6',
        'head',
        'hr',
        'i',
        'img',
        'label',
        'li',
        'ol',
        'p',
        'pre',
        'span',
        'strong',
        'table',
        'tbody',
        'tfoot',
        'td',
        'th',
        'thead',
        'tr',
        'u',
        'ul',
    ]
    attributes = {
        'a': ['class', 'href', 'id', 'style', 'target'],
        'abbr': ['class', 'id', 'style'],
        'acronym': ['class', 'id', 'style'],
        'b': ['class', 'id', 'style'],
        'blockquote': ['class', 'id', 'style'],
        'br': ['class', 'id', 'style'],
        'caption': ['class', 'id', 'style'],
        'center': ['class', 'id', 'style'],
        'code': ['class', 'id', 'style'],
        'em': ['class', 'id', 'style'],
        'div': ['class', 'id', 'style', 'align', 'dir'],
        'font': ['class', 'id', 'style', 'color', 'face', 'size'],
        'h1': ['class', 'id', 'style', 'align', 'dir'],
        'h2': ['class', 'id', 'style', 'align', 'dir'],
        'h3': ['class', 'id', 'style', 'align', 'dir'],
        'h4': ['class', 'id', 'style', 'align', 'dir'],
        'h5': ['class', 'id', 'style', 'align', 'dir'],
        'h6': ['class', 'id', 'style', 'align', 'dir'],
        'head': ['dir', 'lang'],
        'hr': ['align', 'size', 'width'],
        'i': ['class', 'id', 'style'],
        'img': ['class', 'id', 'style', 'align', 'border', 'height', 'hspace', 'src', 'usemap', 'vspace', 'width'],
        'label': ['class', 'id', 'style'],
        'li': ['class', 'id', 'style', 'dir', 'type'],
        'ol': ['class', 'id', 'style', 'dir', 'type'],
        'p': ['class', 'id', 'style', 'align', 'dir'],
        'pre': ['class', 'id', 'style'],
        'span': ['class', 'id', 'style'],
        'strong': ['class', 'id', 'style'],
        'table': [
            'class',
            'id',
            'style',
            'align',
            'bgcolor',
            'border',
            'cellpadding',
            'cellspacing',
            'dir',
            'frame',
            'rules',
            'width',
        ],
        'tbody': ['class', 'id', 'style'],
        'tfoot': ['class', 'id', 'style'],
        'td': [
            'class',
            'id',
            'style',
            'abbr',
            'align',
            'bgcolor',
            'colspan',
            'dir',
            'height',
            'lang',
            'rowspan',
            'scope',
            'style',
            'valign',
            'width',
        ],
        'th': [
            'class',
            'id',
            'style',
            'abbr',
            'align',
            'background',
            'bgcolor',
            'colspan',
            'dir',
            'height',
            'lang',
            'scope',
            'style',
            'valign',
            'width',
        ],
        'thead': ['class', 'id', 'style'],
        'tr': ['class', 'id', 'style', 'align', 'bgcolor', 'dir', 'style', 'valign'],
        'u': ['class', 'id', 'style'],
        'ul': ['class', 'id', 'style', 'dir', 'type'],
    }
    try:
        from bleach.css_sanitizer import CSSSanitizer

        css_sanitizer = CSSSanitizer(
            allowed_css_properties=styles,
        )
        clean_html = lambda body: mark_safe(
            bleach.clean(
                body,
                tags=tags,
                attributes=attributes,
                strip=True,
                strip_comments=True,
                css_sanitizer=css_sanitizer,
            )
        )
    except ModuleNotFoundError:
        # if bleach version is prior to 5.0.0
        clean_html = lambda body: mark_safe(
            bleach.clean(
                body,
                tags=tags,
                attributes=attributes,
                strip=True,
                strip_comments=True,
                styles=styles,
            )
        )
